# -*- coding: utf-8 -*-
'''Nel gioco "chi la spara più grossa" si sfidano due concorrenti A e
B che generano delle sequenze di valori di lunghezza variabile,
rappresentati da un singolo carattere. Le sequenze possono essere di
lunghezza diversa poiché i valori possono essere separati da uno (o
più) spazi bianchi e tab ('\t'). Il numero di caratteri non spazio è,
comunque, uguale per ogni sequenza.

Ogni elemento della sequenza di A viene confrontato con l'elemento
corrispondente della sequenza di B e viene assegnato un punto
- al concorrente che ha generato il valore più alto (per esempio A),
  se la differenza fra il valore di A e il valore di B è inferiore o
  uguale ad un parametro k deciso all'inizio della sfida
- al concorrente che ha generato il valore più basso (per esempio B),
  se la differenza fra il valore di A e il valore di B è superiore
  a k (cioè A ha sballato)
- a nessuno, in caso di pareggio.
Al termine dell'assegnazione, vince chi ha ottenuto più punti. In caso
di pareggio, vince il giocatore che ha generato la sequenza con somma
totale dei valori inferiore.  In caso di ulteriore pareggio, il punto
è assegnato al giocatore con la prima sequenza in ordine
lessicografico. Non può capitare che due giocatori generino
esattamente la stessa sequenza di valori.

Si deve realizzare una funzione che prende in input il parametro k e
una lista di stringhe corrispondenti a un torneo di "chi la spara più
grossa" e restituisce la classifica finale del torneo. La stringa in
posizione i corrisponde alla sequenza dei valori generati dal
giocatore i.

Nel torneo, ogni giocatore sfida tutti gli altri con la propria
sequenza: ovvero, se ci sono n giocatori, ogni giocatore farà n-1
sfide. Il numero di sfide vinte determina la posizione in
classifica. In caso di parità di sfide vinte, i giocatori sono
ordinati in modo crescente in base alla posizione.

Esempio di partite a chi la spara più grossa fra tre giocatori.
    Se k=2 e la lista è ["aac","ccc","caa"]
        La sfida 0, 1 è vinta da 1 per 2 punti a 0, poiché la
            differenza fra "c" e "a" è inferiore o uguale a 2
        La sfida 0, 2 è un pareggio 1 a 1, le due sequenze hanno somma
            uguale, ma vince 0 perché la sequenza "aac" < "caa".
        La sfida 1, 2 è vinta da 1 per 2 punti a 0, poiché la
            differenza fra "c" e "a" è inferiore o uguale a 2.
        Alla fine 0 ha 1 sfida, 1 ha 2 sfide e 2 ha 0 sfide, per cui
            la classifica finale sarà [1, 0, 2].

    Se k=1 e la lista è ["aac","ccc","caa"]
        La sfida 0, 1 è vinta da 0 per 2 punti a 0, poiché la
            differenza fra "c" e "a" è maggiore di 1.
        La sfida 0, 2 è un pareggio 1 a 1, le due sequenze hanno somma
            uguale, ma vince 0 perché la sequenza "aac" < "caa".
        La sfida 1, 2 è vinta da 2 per 2 punti a 0, poiché la
            differenza fra "c" e "a" è maggiore di 1.
        Alla fine 0 ha 2 sfide, 1 ha 0 sfide e 2 ha 1 sfida, per cui
            la classifica finale sarà [0, 2, 1].

    Se k=10 e la lista è  [ "abc",  "dba" , "eZo"]
        La sfida 0, 1 è un pareggio, ma vince 0 perché la sua sequenza
            ha somma inferiore.
        La sfida 0, 2 è vinta da 0 per 2 punti a 1, perché 2 sballa
            con la lettera 'o' contro 'c'.
        La sfida 1, 2 è vinta da 1 per 2 punti a 1, perché 2 sballa
            con la lettera 'o' contro 'a'
        Alla fine 0 ha 2 sfide, 1 ha 1 sfida e 2 ha 0 sfide, per cui
            la classifica finale sarà [0, 1, 2].

    Se k=50 e la lista è  [ "A ƐÈÜ",  "BEAR" , "c Ʈ  ´  ."]
        La sfida 0, 1 è vinta da 1 per 4 punti a 0.
        La sfida 0, 2 è vinta da 2 per 3 punti a 1.
        La sfida 1, 2 è vinta da 1 per 3 punti a 1.
        Alla fine 0 ha 0 sfide, 1 ha 2 sfide e 2 ha 1 sfida, per cui
        la classifica finale sarà [1, 2, 0].

Il timeout per l'esecuzione di ciascun test è di 6 secondi (*2 sualla VM)

'''
# %%
def winner(a, b, A, B):
    a = sum([a for a in A])
    b = sum([a for a in B])
    if a == b:
#        print('in winner with extra tie')
        return A < B
#    print('in winner without extra tie')
    return a<b
# %%

def single_set(a, b, k):
    if abs(a-b)<=k:
        return a>b
    return b>a


def count_tie(A, B):
    return sum(a==b for a, b in zip(A, B))

def single_match(A, B, k):
    a = sum([single_set(a, b, k) for a, b in zip(A, B)])
    b = len(A) - a - count_tie(A, B)
    if a == b:
        return winner (a, b, A, B)
    return a > b


def play_tournament(m, k):
    points = [(0, 0)]*len(m)
    for i in range(len(m)-1):
        for j in range(i+1, len(m)):
            if single_match(m[i], m[j], k):
                w, l = i, j
            else:
                w, l = j, i
            points[w] = w, points[w][1]+1
            points[l] = l, points[l][1]
    return points


def ex(matches, k):
    # Inserisci qui il tuo codice
    m = [0] * len(matches)
    for i in range(len(matches)):
        m[i] = matches[i].replace("\t", "").replace(" ", "")
        m[i] = list(map(ord, list(m[i])))
#    m = matches
#    m = list(map(lambda x: x.translate({ord(' '):"",ord('\t'):""}), matches))
#    m = [[ord(c) for c in match] for match in m]
    points = play_tournament(m, k)
#    print (points)
    points.sort(key = lambda x: (-x[1] ,x[0]))
#    print (points)
    return [x[0] for x in points]


def generate_player_list_int(length, minchar, maxchar):
    from random import randint
    return [chr(randint(minchar, maxchar)) for _ in range(length)]


def generate_player_list_set(length, charset):
    from random import choice
    if type(charset) != tuple:
        charset = tuple(charset)
    return [chr(choice(charset)) for _ in range(length)]


def spaceify(string, space_coefficient):
    space = [' ', '\t']
    if space_coefficient > 0:
        from math import ceil
        from random import randint
        n = ceil(len(string) * space_coefficient)
        for _ in range(n):
            string.insert(randint(0,len(string)), space[randint(0,1)])
    return string


def stringify(string):
    return "".join(string)


def generate_tournament(players, length, minchar, maxchar, space_coefficient):
    tournament = [generate_player_list_int(length, minchar, maxchar) for _ in range(players)]
    tournament = (stringify(spaceify(player, space_coefficient)) for player in tournament)
    return list(tournament)


def generate_tournament_set(players, length, charset, space_coefficient):
    tournament = [generate_player_list_set(length, charset) for _ in range(players)]
    tournament = (stringify(spaceify(player, space_coefficient)) for player in tournament)
    return list(tournament)


def generate_json_test_int(name, players, length, minchar, maxchar, space_coefficient, k):
    import json
    tournament = generate_tournament(players, length, minchar, maxchar, space_coefficient)
    expected = ex(tournament, k)
    testname = 'test_{}'.format(name)
    test = {testname:{'matches':tournament, 'k':k, 'expected': expected}}
#    with open(f'{testname}.json', 'w') as fp:
#        json.dump(test, fp, ensure_ascii=False)

# %%
def generate_json_test_set(name, players, length, charset, space_coefficient, k):
    import json
    tournament = generate_tournament_set(players, length, charset, space_coefficient)
    z = tournament.copy()
    expected = ex(z, k)
    testname = 'test_{}'.format(name)
    test = {testname:{'matches':tournament, 'k':k, 'expected': expected}}
    with open(f'{testname}.json', 'w') as fp:
        json.dump(test, fp, ensure_ascii=False, indent=1)


# %%
if __name__ == "__main__":
    import json
    # Inserisci qui i tuoi test
    hiragana_set = tuple(x for x in range(0x3041,0x309F) if x not in (0x3097,0x3098))
    with open('emojis.json') as f:
        emoji_set = json.load(f)
    emojismall_set = [e for e in emoji_set if e < 11000]
    joyokanji_set = range(0x4e00,0x9fff)
    runic_set = range(0x16A0,0x16F0)

    print([0, 1, 2] == ex([ "abc",  "dba" , "eZo"], 10))
    print([0, 1, 2] == ex([ "a    b c",  "d   b a\t\t\t  \t" , "\te\tZ\to"], 10))
    print([1, 2, 0] == ex(["A ƐÈÜ",  "BEAR" , "c Ʈ  ´  ."], 50))
    print([3, 0, 1, 2, 5, 4] == ex([ "eee",  "ddd" , "ccc", "fff", "aaa", "bbb"], 10))
    print([0, 3, 1, 2, 4, 5] == ex([ "eee",  "ddd" , "ccc", "fff", "aaa", "bbb"], 5))
