# -*- coding: utf-8 -*-
'''Nel gioco "chi la spara più grossa" si sfidano due concorrenti A e
B che generano delle sequenze di valori di lunghezza variabile,
rappresentati da un singolo carattere. Le sequenze possono essere di
lunghezza diversa poiché i valori possono essere separati da uno (o
più) spazi bianchi e tab ('\t'). Il numero di caratteri non spazio è,
comunque, uguale per ogni sequenza.

Ogni elemento della sequenza di A viene confrontato con l'elemento
corrispondente della sequenza di B e viene assegnato un punto
- al concorrente che ha generato il valore più alto (per esempio A),
  se la differenza fra il valore di A e il valore di B è inferiore o
  uguale ad un parametro k deciso all'inizio della sfida
- al concorrente che ha generato il valore più basso (per esempio B),
  se la differenza fra il valore di A e il valore di B è superiore
  a k (cioè A ha sballato)
- a nessuno, in caso di pareggio.
Al termine dell'assegnazione, vince chi ha ottenuto più punti. In caso
di pareggio, vince il giocatore che ha generato la sequenza con somma
totale dei valori inferiore.  In caso di ulteriore pareggio, il punto
è assegnato al giocatore con la prima sequenza in ordine
lessicografico. Non può capitare che due giocatori generino
esattamente la stessa sequenza di valori.

Si deve realizzare una funzione che prende in input il parametro k e
una lista di stringhe corrispondenti a un torneo di "chi la spara più
grossa" e restituisce la classifica finale del torneo. La stringa in
posizione i corrisponde alla sequenza dei valori generati dal
giocatore i.

Nel torneo, ogni giocatore sfida tutti gli altri con la propria
sequenza: ovvero, se ci sono n giocatori, ogni giocatore farà n-1
sfide. Il numero di sfide vinte determina la posizione in
classifica. In caso di parità di sfide vinte, i giocatori sono
ordinati in modo crescente in base alla posizione.

Esempio di partite a chi la spara più grossa fra tre giocatori.
    Se k=2 e la lista è ["aac","ccc","caa"]
        La sfida 0, 1 è vinta da 1 per 2 punti a 0, poiché la
            differenza fra "c" e "a" è inferiore o uguale a 2
        La sfida 0, 2 è un pareggio 1 a 1, le due sequenze hanno somma
            uguale, ma vince 0 perché la sequenza "aac" < "caa".
        La sfida 1, 2 è vinta da 1 per 2 punti a 0, poiché la
            differenza fra "c" e "a" è inferiore o uguale a 2.
        Alla fine 0 ha 1 sfida, 1 ha 2 sfide e 2 ha 0 sfide, per cui
            la classifica finale sarà [1, 0, 2].

    Se k=1 e la lista è ["aac","ccc","caa"]
        La sfida 0, 1 è vinta da 0 per 2 punti a 0, poiché la
            differenza fra "c" e "a" è maggiore di 1.
        La sfida 0, 2 è un pareggio 1 a 1, le due sequenze hanno somma
            uguale, ma vince 0 perché la sequenza "aac" < "caa".
        La sfida 1, 2 è vinta da 2 per 2 punti a 0, poiché la
            differenza fra "c" e "a" è maggiore di 1.
        Alla fine 0 ha 2 sfide, 1 ha 0 sfide e 2 ha 1 sfida, per cui
            la classifica finale sarà [0, 2, 1].

    Se k=10 e la lista è  [ "abc",  "dba" , "eZo"]
        La sfida 0, 1 è un pareggio, ma vince 0 perché la sua sequenza
            ha somma inferiore.
        La sfida 0, 2 è vinta da 0 per 2 punti a 1, perché 2 sballa
            con la lettera 'o' contro 'c'.
        La sfida 1, 2 è vinta da 1 per 2 punti a 1, perché 2 sballa
            con la lettera 'o' contro 'a'
        Alla fine 0 ha 2 sfide, 1 ha 1 sfida e 2 ha 0 sfide, per cui
            la classifica finale sarà [0, 1, 2].

    Se k=50 e la lista è  [ "A ƐÈÜ",  "BEAR" , "c Ʈ  ´  ."]
        La sfida 0, 1 è vinta da 1 per 4 punti a 0.
        La sfida 0, 2 è vinta da 2 per 3 punti a 1.
        La sfida 1, 2 è vinta da 1 per 3 punti a 1.
        Alla fine 0 ha 0 sfide, 1 ha 1 sfida e 2 ha 2 sfide, per cui
        la classifica finale sarà [2, 1, 0].

'''
def mano(o1,o2,k):
    diff = abs(o1-o2)
    if diff==0:             # se caratteri uguali 
        return 0            # patta
    elif diff <= k:         # se diff <= k
        assert o1!=o2, "o1 == o2"
        if o1>o2:           # 1 punto al maggiore
            return -1       # vince player1
        else:
            return +1       # vince player2
    else:                   # se diff < k
        assert o1!=o2, "o1 == o2"
        if o1>o2:           # 1 punto al minore
            return +1       # vince player2
        else:
            return -1       # vince player1


def partita(str1, str2, k):
    punti1 = punti2 = 0
    S1 = S2 = 0
    for c1, c2 in zip(str1, str2):
        o1, o2 = ord(c1), ord(c2)
        S1 += o1
        S2 += o2
        m = mano(o1,o2,k)
        if m == -1:
            punti1 += 1
        elif m == 1:
            punti2 += 1
    if punti1 == punti2:    # se parità
        if S1<S2:           # se S1<S2
            return -1       # vince chi ha S minore (player1)
        elif S1>S2:
            return 1        # player2
        else:
            assert str1 != str2, "Le due stringhe non possono essere uguali"
            if str1 < str2:        # se S1==S2 si guarda l'ordine lessicografico
                return -1       # vince player1
            else:
                return +1       # vince player2
    elif punti1 > punti2:
        return -1
    else:
        return +1

def ex(matches, k):
    # Inserisci qui il tuo codice
    N = len(matches)
    for i in range(N): 
        matches[i] = matches[i].replace(' ','')
        matches[i] = matches[i].replace('\t','')
    punti = [0]*N
    for g1,v1 in enumerate(matches[:-1]):
        for g2,v2 in enumerate(matches[g1+1:]):
            G2 = g2+g1+1
            vince = partita(v1, v2, k)
            print('tra\t', g1,v1, G2,v2, '\tvince', vince, (v1 if vince<0 else (v2 if vince>0 else 'parità')))
            if vince == 0:      # se parità si passa alla prox coppia
                continue
            elif vince < 0:     # se vince player1
                punti[g1] += 1  # si incrementa il suo punteggio
            else:
                punti[G2] += 1  # altrimenti quello di player2
    classifica = dict(enumerate(punti))
    print('punti      ', classifica)
    ordinata   = sorted( classifica.items(), reverse=True, key=lambda x: x[1] )
    print('classifica ', ordinata)
    graduatoria = [ k for k,_ in ordinata ]
    print('graduatoria', graduatoria)
    return graduatoria


if __name__ == "__main__":
    # Inserisci qui i tuoi test
    k        = 2
    matches  = ["aac","ccc","caa"]
    expected = [1, 0, 2]
    print(" 0 ha 1 sfida, 1 ha 2 sfide e 2 ha 0 sfide")
    try:
        assert ex(matches, k) == expected, "Test 1 FAILED"
    except Exception as e:
        print(e)
    k        = 1
    matches  = ["aac","ccc","caa"]
    expected = [0, 2, 1]
    print(" 0 ha 2 sfida, 1 ha 0 sfide e 2 ha 1 sfide")
    try:
        assert ex(matches, k) == expected, "Test 2 FAILED"
    except Exception as e:
        print(e)
    k        = 10 
    matches  = [ "abc",  "dba" , "eZo"]
    expected = [0, 1, 2]
    print(" 0 ha 2 sfida, 1 ha 1 sfide e 2 ha 0 sfide")
    try:
        assert ex(matches, k) == expected, "Test 3 FAILED"
    except Exception as e:
        print(e)
    k        = 50 
    matches  = [ "A ƐÈÜ",  "BEAR" , "c Ʈ  ´  ."]
    expected = [1, 2, 0]
    print(" 0 ha 0 sfida, 1 ha 1 sfide e 2 ha 2 sfide")
    try:
        assert ex(matches, k) == expected, "Test 4 FAILED"
    except Exception as e:
        print(e)

'''
OUTPUT
 0 ha 1 sfida, 1 ha 2 sfide e 2 ha 0 sfide
tra	 0 aac 1 ccc 	vince 1 ccc
tra	 0 aac 2 caa 	vince -1 aac
tra	 1 ccc 2 caa 	vince -1 ccc
punti       {0: 1, 1: 2, 2: 0}
classifica  [(1, 2), (0, 1), (2, 0)]
graduatoria [1, 0, 2]
 0 ha 2 sfida, 1 ha 0 sfide e 2 ha 1 sfide
tra	 0 aac 1 ccc 	vince -1 aac
tra	 0 aac 2 caa 	vince -1 aac
tra	 1 ccc 2 caa 	vince 1 caa
punti       {0: 2, 1: 0, 2: 1}
classifica  [(0, 2), (2, 1), (1, 0)]
graduatoria [0, 2, 1]
 0 ha 2 sfida, 1 ha 1 sfide e 2 ha 0 sfide
tra	 0 abc 1 dba 	vince -1 abc
tra	 0 abc 2 eZo 	vince -1 abc
tra	 1 dba 2 eZo 	vince -1 dba
punti       {0: 2, 1: 1, 2: 0}
classifica  [(0, 2), (1, 1), (2, 0)]
graduatoria [0, 1, 2]
 0 ha 0 sfida, 1 ha 1 sfide e 2 ha 2 sfide
tra	 0 AƐÈÜ 1 BEAR 	vince 1 BEAR
tra	 0 AƐÈÜ 2 cƮ´. 	vince 1 cƮ´.
tra	 1 BEAR 2 cƮ´. 	vince -1 BEAR
punti       {0: 0, 1: 2, 2: 1}
classifica  [(1, 2), (2, 1), (0, 0)]
graduatoria [1, 2, 0]
Test 4 FAILED
'''
